import { Promise as Promise$1 } from 'bluebird';
import { get } from 'lodash';
import * as _ from 'lodash';
import { Client } from 'elasticsearch';
import * as elasticsearch from 'elasticsearch';
import { ErrorType, HttpStatusCode } from 'azure-functions-ts-essentials';

const ELASTICSEARCH_REFRESH = 'wait_for';
const ELASTICSEARCH_MAX_RESULTS = 2147483519;
const getElasticClient = (host) => new Client({
    host,
    defer: () => Promise$1.defer()
});
const getStringDate = (now) => {
    if (!now)
        now = new Date();
    const year = now.getUTCFullYear();
    let month = (now.getUTCMonth() + 1).toString();
    let day = now.getUTCDate().toString();
    month = `${Number(month) < 10 ? '0' : ''}${month}`;
    day = `${Number(day) < 10 ? '0' : ''}${day}`;
    return `${year}.${month}.${day}`;
};
const getErrorResponse = (err) => {
    if (Array.isArray(err) && err.length > 0)
        return {
            status: HttpStatusCode.BadRequest,
            body: err.map(cur => ({
                type: get(cur, 'type'),
                message: get(cur, 'reason'),
                index: get(cur, 'index'),
                uuid: get(cur, 'index_uuid')
            }))
        };
    return {
        status: err.status,
        body: {
            type: err.displayName,
            message: err.message
        }
    };
};
class Elasticizer {
    constructor(host, type, options) {
        this.host = host;
        this.type = type;
        this.options = options;
        this.client = getElasticClient(host);
        this.refresh = get(options, 'refresh', ELASTICSEARCH_REFRESH);
        this.prefix = get(options, 'prefix', '');
    }
    getOne(index, id) {
        const query$ = this.client.get({
            index: `${this.prefix}${index}`,
            type: this.type,
            id
        });
        return query$
            .then((res) => ({
            status: HttpStatusCode.OK,
            body: Object.assign({ _index: res._index, _id: res._id }, res._source)
        }))
            .catch(getErrorResponse);
    }
    search(index, body, query, page, perPage, sortAsc) {
        const query$ = this.client.search({
            index: Array.isArray(index)
                ? index.map(cur => `${this.prefix}${cur}`)
                : `${this.prefix}${index}`,
            body,
            q: query,
            from: Number(page) >= 0 && Number(perPage) > 0 ? Number(page) * Number(perPage) : 0,
            size: Number(perPage) > 0 ? Number(perPage) : ELASTICSEARCH_MAX_RESULTS,
            sort: `createdAtUtc:${sortAsc ? 'asc' : 'desc'}`
        });
        return query$
            .then((res) => ({
            status: HttpStatusCode.OK,
            body: {
                data: res.hits.hits.map((cur) => (Object.assign({ _index: cur._index, _id: cur._id }, cur._source))),
                hasMore: Number(page) >= 0 && Number(perPage) > 0
                    ? res.hits.total > (Number(page) + 1) * Number(perPage)
                    : false,
                totalCount: res.hits.total
            }
        }))
            .catch(getErrorResponse);
    }
    insertMany(req) {
        const contentType = req.headers ? req.headers['content-type'] : undefined;
        if (!(contentType && contentType.indexOf('application/json') >= 0))
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Invalid
                }
            });
        if (!(req.body && Object.keys(req.body).length))
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Invalid
                }
            });
        const query$ = this.client.bulk({
            type: this.type,
            refresh: this.refresh,
            body: req.body.reduce((acc, cur) => {
                const index = { _index: `${this.prefix}${cur._index}` };
                delete cur._index;
                acc.push({ index: Object.assign({}, index) }, Object.assign({}, cur, { createdAtUtc: new Date().toISOString() }));
                return acc;
            }, [])
        });
        return query$
            .then((res) => {
            const isError = get(res, 'errors');
            if (isError)
                return getErrorResponse(res.items.map((cur) => cur.index.error));
            return {
                status: HttpStatusCode.Created,
                body: {}
            };
        })
            .catch(getErrorResponse);
    }
    updateOne(index, req, id) {
        const contentType = req.headers ? req.headers['content-type'] : undefined;
        if (!(contentType && contentType.indexOf('application/json') >= 0))
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Invalid
                }
            });
        if (!(req.body && Object.keys(req.body).length))
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Invalid
                }
            });
        if (!id)
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Invalid
                }
            });
        delete req.body._index;
        const query$ = this.client.update({
            index: `${this.prefix}${index}`,
            type: this.type,
            refresh: this.refresh,
            id,
            body: {
                doc: req.body
            }
        });
        return query$
            .then(() => ({
            status: HttpStatusCode.OK,
            body: {}
        }))
            .catch(getErrorResponse);
    }
    deleteOne(index, id) {
        if (!id)
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Missing
                }
            });
        const query$ = this.client.delete({
            index: `${this.prefix}${index}`,
            type: this.type,
            refresh: this.refresh,
            id
        });
        return query$
            .then(() => ({
            status: HttpStatusCode.OK,
            body: {}
        }))
            .catch(getErrorResponse);
    }
}

export { ELASTICSEARCH_REFRESH, ELASTICSEARCH_MAX_RESULTS, getElasticClient, getStringDate, Elasticizer };
//# sourceMappingURL=azure-functions-elasticizer.js.map
